//  @license
// --------------------------------------------------------------------------
//  Copyright (C) 2015-2020 Virgil Security, Inc.
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//
//      (1) Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//      (2) Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in
//      the documentation and/or other materials provided with the
//      distribution.
//
//      (3) Neither the name of the copyright holder nor the names of its
//      contributors may be used to endorse or promote products derived from
//      this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY EXPRESS OR
//  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
//  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
//  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//  POSSIBILITY OF SUCH DAMAGE.
//
//  Lead Maintainer: Virgil Security Inc. <support@virgilsecurity.com>
// --------------------------------------------------------------------------
// clang-format off


//  @warning
// --------------------------------------------------------------------------
//  This file is partially generated.
//  Generated blocks are enclosed between tags [@<tag>, @end].
//  User's code can be added between tags [@end, @<tag>].
// --------------------------------------------------------------------------


//  @description
// --------------------------------------------------------------------------
//  Create a bridge between "raw keys" and algorithms that can import them.
// --------------------------------------------------------------------------

#ifndef VSCF_KEY_ALG_FACTORY_H_INCLUDED
#define VSCF_KEY_ALG_FACTORY_H_INCLUDED

#include "vscf_library.h"
#include "vscf_error.h"
#include "vscf_alg_id.h"
#include "vscf_impl.h"
#include "vscf_raw_public_key.h"
#include "vscf_raw_private_key.h"

// clang-format on
//  @end


#ifdef __cplusplus
extern "C" {
#endif


//  @generated
// --------------------------------------------------------------------------
// clang-format off
//  Generated section start.
// --------------------------------------------------------------------------

//
//  Create a key algorithm based on an identifier.
//
VSCF_PUBLIC vscf_impl_t *
vscf_key_alg_factory_create_from_alg_id(vscf_alg_id_t alg_id, const vscf_impl_t *random, vscf_error_t *error);

//
//  Create a key algorithm correspond to a specific key.
//
VSCF_PUBLIC vscf_impl_t *
vscf_key_alg_factory_create_from_key(const vscf_impl_t *key, const vscf_impl_t *random, vscf_error_t *error);

//
//  Create a key algorithm that can import "raw public key".
//
VSCF_PUBLIC vscf_impl_t *
vscf_key_alg_factory_create_from_raw_public_key(const vscf_raw_public_key_t *public_key, const vscf_impl_t *random,
        vscf_error_t *error);

//
//  Create a key algorithm that can import "raw private key".
//
VSCF_PUBLIC vscf_impl_t *
vscf_key_alg_factory_create_from_raw_private_key(const vscf_raw_private_key_t *private_key, const vscf_impl_t *random,
        vscf_error_t *error);


// --------------------------------------------------------------------------
//  Generated section end.
// clang-format on
// --------------------------------------------------------------------------
//  @end


#ifdef __cplusplus
}
#endif


//  @footer
#endif // VSCF_KEY_ALG_FACTORY_H_INCLUDED
//  @end
